;###########################################################################
;###########################################################################
; ABOUT Shapes:
;
;	This code module contains all of the functions that relate to 
;	the tetris shapes. The functions in here manipulate, create, 
;	destroy, and perform all other necessary handling.
;
;
;###########################################################################
;###########################################################################

;###########################################################################
;###########################################################################
; THE COMPILER OPTIONS
;###########################################################################
;###########################################################################

	.386
	.model flat, stdcall
	option casemap :none   ; case sensitive

;###########################################################################
;###########################################################################
; THE INCLUDES SECTION
;###########################################################################
;###########################################################################

	;================================================
	; These are the Inlcude files for Window stuff
	;================================================
	include \masm32\include\windows.inc
	include \masm32\include\comctl32.inc
	include \masm32\include\comdlg32.inc
	include \masm32\include\shell32.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc
	include \masm32\include\winmm.inc
	
	;===============================================
	; The Lib's for those included files
	;================================================
	includelib \masm32\lib\comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib
	includelib \masm32\lib\winmm.lib

	;=================================================
	; Include the Direct Draw Library
	;=================================================
	include Includes/DDRaw.inc

	;=================================================
	; Include the file that has our protos
	;=================================================
	include Protos.inc

;###########################################################################
;###########################################################################
; LOCAL MACROS
;###########################################################################
;###########################################################################

	m2m MACRO M1, M2
		push		M2
		pop		M1
	ENDM

	return MACRO arg
		mov	eax, arg
        	ret
	ENDM

	RGB MACRO red, green, blue
		xor	eax,eax
		mov	ah,blue
		shl	eax,8
		mov	ah,green
		mov	al,red
	ENDM

;#################################################################################
;#################################################################################
; External variables
;#################################################################################
;#################################################################################

	;=================================
	; The DirectDraw stuff
	;=================================
	EXTERN	lpddsprimary	:LPDIRECTDRAWSURFACE4
	EXTERN	lpddsback	:LPDIRECTDRAWSURFACE4
	EXTERN	Is_555		:BYTE

;#################################################################################
;#################################################################################
; The DATA Section
;#################################################################################
;#################################################################################

  .data
	;==============================
	; For the Cur Shape
	;==============================
	CurShape	dd 0	
	CurShapeColor	dd 0
	CurShapeX	dd 0
	CurShapeY	dd 0
	CurShapeFrame	dd 0

	;=============================
	; For the next shape up
	;=============================
	NextShape	dd 0
	NextShapeColor	dd 0
	NextShapeX	dd 0
	NextShapeY	dd 0
	NextShapeFrame	dd 0

	;==============================
	; For the Source during a blit
	;==============================
	SrcRect	RECT	<0,0,24,24>

	;==============================
	; Array to hold 8 DD surfaces
	;==============================
	BlockSurface	LPDIRECTDRAWSURFACE4 8 dup (0)
	
	;==============================
	; Pitch for the surfaces
	;==============================
	lPitch		dd 0

	;==============================
	; This is our generic string
	; and our buffer string for
	; the wvsprintf call
	;==============================
	szBlocks	db "Art\Block%d.sfp",0
	szBuffer	db 18 dup (0)	; This creates a buffer of size 
					; 18 filled with 0's dup = duplicate
	dwArgs		dd 4 dup (0)	; Array of 4 DWORDS for the wvsprintf call
	
	;===============================
	; For their score and level
	;===============================
	Score		dd 0
	CurLevel	dd 0
	NumLines	dd 0
	szLevel		db "%02d",0
	szLines		db "%02d",0
	szScore		db "%06d",0
	szImpact	db "Impact",0

	;========================================
	; Rectangle structure for caption drawing
	;========================================
	text_rect	RECT <?>

	;========================================
	; A DC for the drawing of the captions
	;========================================
	hDC		dd 0
	Old_Obj		dd 0

	;================================
	; Our very cool sounds
	;================================
	szThudSnd	db "Sound\Thud.wav",0
	szDropSnd	db "Sound\Drop.wav",0
	szRotateSnd	db "Sound\Rotate.wav",0

	;===============================
	; ID for the sounds
	;===============================
	Thud_ID		dd 0
	Drop_ID		dd 0
	Rotate_ID	dd 0

	;===================================================
	; This is a pointer to the grid table
	;
	; The coords (0,0) are the first byte and the 
	; coords (13,20) are the last byte in the memory
	;===================================================
	BlockGrid	dd 0

	;===================================================
	; Here is our shape table it contains every possible
	; combination of values for the different shapes
	; in order to give us the correct shape for every
	; possible rotation.
	;===================================================
ShapeTable	\			; Here is our square
		\	
	db	00000000b		; Position 1
	db	00000110b	
	db	00000110b	
	db	00000000b	
			
	db	00000000b		; Position 2
	db	00000110b	
	db	00000110b	
	db	00000000b	
			
	db	00000000b		; Position 3
	db	00000110b	
	db	00000110b	
	db	00000000b	
			
	db	00000000b		; Position 4
	db	00000110b	
	db	00000110b	
	db	00000000b	
				; Here is our Line
			
Line	db	00001000b		; Position 1
	db	00001000b	
	db	00001000b	
	db	00001000b		
			
	db	00000000b		; Position 2
	db	00000000b	
	db	00000000b	
	db	00001111b		
			
	db	00000001b		; Position 3
	db	00000001b	
	db	00000001b	
	db	00000001b		
			
	db	00001111b		; Position 4
	db	00000000b	
	db	00000000b	
	db	00000000b		
				; Here is our Pyramid
			
Pyramid	db	00001110b		; Position 1
	db	00000100b	
	db	00000000b	
	db	00000000b		
			
	db	00001000b		; Position 2
	db	00001100b	
	db	00001000b	
	db	00000000b		
			
	db	00000000b		; Position 3
	db	00000100b	
	db	00001110b	
	db	00000000b		
			
	db	00000001b		; Position 4
	db	00000011b	
	db	00000001b	
	db	00000000b		
				; Here is our L
			
L	db	00001000b		; Position 1
	db	00001000b	
	db	00001100b	
	db	00000000b		
			
	db	00000000b		; Position 2
	db	00000010b	
	db	00001110b	
	db	00000000b		
			
	db	00000110b		; Position 3
	db	00000010b	
	db	00000010b	
	db	00000000b		
			
	db	00001110b		; Position 4
	db	00001000b	
	db	00000000b	
	db	00000000b		
				; Here is our Backwards L
			
Back_L	db	00001100b		; Position 1
	db	00001000b	
	db	00001000b	
	db	00000000b		
			
	db	00000000b		; Position 2
	db	00001000b	
	db	00001110b	
	db	00000000b		
			
	db	00000001b		; Position 3
	db	00000001b	
	db	00000011b	
	db	00000000b		
			
	db	00001110b		; Position 4
	db	00000010b	
	db	00000000b	
	db	00000000b		
				; Here is our Backwards Z
			
Back_Z	db	00000100b		; Position 1
	db	00000110b	
	db	00000010b	
	db	00000000b		
			
	db	00000110b		; Position 2
	db	00001100b	
	db	00000000b	
	db	00000000b		
			
	db	00000100b		; Position 3
	db	00000110b	
	db	00000010b	
	db	00000000b		
			
	db	00000110b		; Position 4
	db	00001100b	
	db	00000000b	
	db	00000000b		
				; Here is our Z
			
Z	db	00000010b		; Position 1
	db	00000110b	
	db	00000100b	
	db	00000000b		
			
	db	00001100b		; Position 2
	db	00000110b	
	db	00000000b	
	db	00000000b		
			
	db	00000010b		; Position 3
	db	00000110b	
	db	00000100b	
	db	00000000b		
			
	db	00001100b		; Position 4
	db	00000110b	
	db	00000000b	
	db	00000000b	

	;===================================
	; Make the square shape the
	; same address as our shape table
	;===================================
Square	equ	ShapeTable

;#################################################################################
;#################################################################################
; BEGIN EQUATES
;#################################################################################
;#################################################################################

	;=================
	;Utility Equates
	;=================
FALSE		equ	0
TRUE		equ	1

	;=================
	; The Screen BPP
	;=================
screen_bpp	equ	16

	;=================
	; Block Dimensions
	;=================
BLOCK_WIDTH	equ	24
BLOCK_HEIGHT	equ	24

	;=================
	; Grid Dimensions
	;=================
GRID_WIDTH	equ	13
GRID_HEIGHT	equ	24	; 4 Taller than the highest so
				; we can fit the new piece above
				; the top and have it drop down

	;=================
	; Movement EQU's
	;=================
MOVE_LEFT	equ	1
MOVE_RIGHT	equ	2
MOVE_DOWN	equ	3

	;=============================
	; Max score, lines, and level
	;=============================
MAX_SCORE	equ	999999
MAX_LEVEL	equ	10
MAX_LINES	equ	20

;#################################################################################
;#################################################################################
; BEGIN THE CODE SECTION
;#################################################################################
;#################################################################################

  .code

;########################################################################
; Init_Shapes Procedure
;########################################################################
Init_Shapes	proc	

	;=======================================================
	; This function will perform the needed initialization
	; for all of our shapes
	;=======================================================

	;==========================
	; Local Variables
	;==========================
	LOCAL	ptr_BMP		:DWORD
	LOCAL	number		:DWORD

	;=================================
	; Set the starting number to one
	;=================================
	mov	number, 1

	;=================================
	; Loop through all 8 blocks
	;=================================
	.while number <= 8
		;=============================
		; Create the surface for the
		; current block
		;=============================
		invoke DD_Create_Surface, BLOCK_WIDTH, BLOCK_HEIGHT, NULL
		push	eax
		mov	eax, number
		dec	eax
		pop	ebx
		shl	eax, 2
		mov	DWORD PTR BlockSurface[eax], ebx

		;=============================
		; Check for an error
		;=============================
		.if ebx == NULL
			jmp	err
		.endif

		;=============================
		; Get the name of the bitmap
		; using wvsprintf
		;=============================
		m2m	dwArgs, number
		invoke wvsprintfA, ADDR szBuffer, ADDR szBlocks, offset dwArgs

		;===============================
		; Now that we have the filename
		; load it into memory
		;===============================
		invoke Create_From_SFP, ADDR ptr_BMP, ADDR szBuffer, screen_bpp

		;====================================
		; Test for an error
		;====================================
		.if eax == FALSE
			;========================
			; We failed so leave
			;========================
			jmp	err
	
		.endif

		;===============================
		; Load the bitmap into the
		; block surface
		;===============================
		mov	eax, number
		dec	eax
		shl	eax, 2
		mov	ebx, DWORD PTR BlockSurface[eax ]
		invoke DD_Load_Bitmap, ebx, ptr_BMP, \
			BLOCK_WIDTH, BLOCK_HEIGHT, screen_bpp

		;============================
		; Check for an error
		;============================
		.if eax == FALSE
			;===================
			; Jump to err
			;===================
			jmp	err

		.endif

		;====================
		; Pause just a bit
		;====================
		invoke Delay_Time, 500

		;===============================
		; Free the bitmap
		;===============================
		invoke GlobalFree, ptr_BMP

		;===============================
		; Increment our number counter
		;===============================
		inc	number

	.endw

	;===============================
	; Allocate Memory for the grid
	;===============================
	invoke GlobalAlloc, GMEM_FIXED, (GRID_WIDTH * GRID_HEIGHT)
	mov	BlockGrid, eax

	;===================================
	; test for an error
	;===================================
	.if eax == 0
		jmp	err
	.endif

	;========================
	; Load in the Thud sound
	;========================
	invoke Load_WAV, ADDR szThudSnd, NULL
	mov	Thud_ID, eax 

	;========================
	; Load in the Drop sound
	;========================
	invoke Load_WAV, ADDR szDropSnd, NULL
	mov	Drop_ID, eax 

	;========================
	; Load the Rotate sound
	;========================
	invoke Load_WAV, ADDR szRotateSnd, NULL
	mov	Rotate_ID, eax 

	;=======================================
	; Set the score, level, and lines to 0
	;=======================================
	mov	Score, 0
	mov	CurLevel, 1
	mov	NumLines, 0

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Init_Shapes	ENDP
;########################################################################
; END Init_Shapes
;########################################################################

;########################################################################
; Shutdown_Shapes Procedure
;########################################################################
Shutdown_Shapes	proc	

	;=======================================================
	; This function will perform the needed shutdown
	; for all of our shapes
	;=======================================================

	;==========================
	; Local Variables
	;==========================
	LOCAL	number		:DWORD

	;=================================
	; Set the starting number to one
	;=================================
	mov	number, 1

	;=================================
	; Loop through all 8 blocks
	;=================================
	.while number <= 8
	
		;=========================
		; Calc the block surface
		; to release
		;=========================
		mov	eax, number
		dec	eax
		shl	eax, 2
		mov	ebx, DWORD PTR BlockSurface[eax]

		;======================
		; Relase the surface
		;======================
		DDS4INVOKE Release, ebx
	
		;===============================
		; Increment our number counter
		;===============================
		inc	number

	.endw

	;=======================
	; Free Grid Memory
	;=======================
	invoke GlobalFree, BlockGrid

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Shutdown_Shapes	ENDP
;########################################################################
; END Shutdown_Shapes
;########################################################################

;########################################################################
; New_Game Procedure
;########################################################################
New_Game	proc	

	;=======================================================
	; This function will reset the variables for the game
	;=======================================================

	;=======================================
	; Set the score, level, and lines to 0
	;=======================================
	mov	Score, 0
	mov	CurLevel, 1
	mov	NumLines, 0
	
done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

New_Game	ENDP
;########################################################################
; END New_Game
;########################################################################

;########################################################################
; Init_Grid Procedure
;########################################################################
Init_Grid	proc	

	;=======================================================
	; This function will perform the needed initialization
	; for our grid and for the current shape and next shape
	;=======================================================

	;===================================
	; Clear out the memory for the grid
	; Set it to all 0's
	;===================================
	invoke RtlFillMemory, BlockGrid, (GRID_WIDTH * GRID_HEIGHT), 0

	;=========================================
	; Set Current and Next shapes to Unloaded
	;=========================================
	mov	CurShape, -1
	mov	NextShape, -1

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Init_Grid	ENDP
;########################################################################
; END Init_Grid
;########################################################################

;########################################################################
; Draw_Shape Procedure
;########################################################################
Draw_Shape	proc	UseNext:BYTE

	;=======================================================
	; This function will draw our current shape at its
	; proper location on the screen or it will draw the next
	; shape on the screen in the next window
	;=======================================================

	;===========================
	; Local Variables
	;===========================
	LOCAL	DrawY:		DWORD
	LOCAL	DrawX:		DWORD
	LOCAL	CurRow:		DWORD
	LOCAL	CurCol:		DWORD
	LOCAL	CurLine:	DWORD
	LOCAL	XPos:		DWORD
	LOCAL	YPos:		DWORD

	;===================================
	; Get the Current Shape Pos
	;===================================
	.if UseNext == FALSE
		mov	ebx, CurShape
		mov	eax, CurShapeFrame
	.else
		mov	ebx, NextShape
		mov	eax, NextShapeFrame
	.endif
	shl	eax, 2
	add	ebx, eax
	mov	CurLine, ebx

	;===================================
	; Set the Starting Row and Column
	; for the drawing
	;===================================
	.if UseNext == FALSE
		mov	eax, CurShapeX
		mov	ebx, CurShapeY
	.else
		mov	eax, 2		; X Coord
		mov	ebx, 4		; Y Coord
	.endif
	mov	DrawX, eax
	mov	DrawY, ebx

	;===================================
	; Loop through all four rows
	;===================================
	mov	CurRow, 0
	.while CurRow < 4
		;=====================================
		; Loop through all four Columns if
		; the Y Coord is in the screen
		;=====================================
		mov	CurCol, 4
		.while CurCol > 0 && DrawY < 20
			;===============================
			; Shift the CurLine Byte over
			; by our CurCol
			;===============================
			mov	ecx, 4
			sub	ecx, CurCol
			mov	ebx, CurLine
			xor	eax, eax
			mov	al, BYTE PTR [ebx]
			shr	eax, cl

			;===============================
			; Is it a valid block?
			;===============================
			.if ( eax & 1 )
				;============================
				; Yes it was a valid block
				;============================
	
				;=============================
				; Calculate the Y coord
				;=============================
				mov	eax, (GRID_HEIGHT - 5)
				sub	eax, DrawY
				mov	ecx, BLOCK_HEIGHT
				mul	ecx
				mov	YPos, eax

				;===========================
				; Adjust the Y coord for
				; certain shapes in the next
				; window since they are off
				; of the center
				;===========================
				.if UseNext == TRUE
					mov	ecx, NextShape
					.if ecx == Offset Square || ecx == Offset Line
						sub	YPos, 7
					.elseif ecx == offset Pyramid
						add	YPos, 15
					.else
						add	YPos, 5
					.endif
				.endif

				;=============================
				; Calculate the X coord
				;=============================
				mov	eax, DrawX
				add	eax, CurCol
				dec	eax
				mov	ecx, BLOCK_WIDTH
				mul	ecx
				.if UseNext == FALSE
					add	eax, 251
				.else
					add	eax, 40
					;=============================
					; Now adjust the X coord on a
					; shape by shape basis
					;=============================
					mov	ecx, NextShape
					.if ecx == offset Square
						sub	eax, 12
					.elseif ecx == offset Line
						add	eax, 25
					.elseif ecx == offset L
						add	eax, 15
					.elseif ecx == offset Back_L
						add	eax, 15
					.elseif ecx == offset Z
						sub	eax, 15
					.elseif ecx == offset Back_Z
						sub	eax, 15
					.endif
				.endif
				mov	XPos, eax

				;=============================
				; Calculate the surface to use
				;=============================
				.if UseNext == FALSE
					mov	eax, CurShapeColor
				.else
					mov	eax, NextShapeColor
				.endif
				shl	eax, 2
				mov	ebx, DWORD PTR BlockSurface[eax]

				;=============================
				; Blit the block
				;=============================
				DDS4INVOKE BltFast, lpddsback, XPos, YPos, \
					ebx, ADDR SrcRect, \
					DDBLTFAST_NOCOLORKEY or DDBLTFAST_WAIT

			.endif

			;=====================
			; Dec our col counter
			;=====================
			dec	CurCol

		.endw

		;=======================
		; Inc the CurLine
		;=======================
		inc	CurLine

		;====================
		; decrement Y coord
		;====================
		dec	DrawY

		;====================
		; Inc the row counter
		;====================
		inc	CurRow

	.endw

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Draw_Shape	ENDP
;########################################################################
; END Draw_Shape
;########################################################################

;########################################################################
; Draw_Grid Procedure
;########################################################################
Draw_Grid	proc	

	;=======================================================
	; This function will draw our grid. If the value is zero
	; there is no block otherwise the value is the block#
	;=======================================================

	;====================
	; Local Variables
	;====================
	LOCAL	CurRow:		DWORD
	LOCAL	CurCol:		DWORD
	LOCAL	CurBlock:	DWORD
	LOCAL	YPos:		DWORD
	LOCAL	XPos:		DWORD

	;============================
	; Start the current block at
	; the beggining of our grid
	;============================
	mov	eax, BlockGrid
	mov	CurBlock, eax

	;============================
	; Initialize the current row
	;============================
	mov	CurRow, 0

	;=============================
	; Loop through all of our rows
	;=============================
	.while CurRow < ( GRID_HEIGHT - 4 )
		
		;================================
		; Initialize the currrent column 
		;================================
		mov	CurCol, 0
	
		;=============================
		; Loop through all of our cols
		;=============================
		.while CurCol < GRID_WIDTH
			;========================
			; Is there a Block here
			;========================
			xor	eax, eax
			mov	ebx, CurBlock
			mov	al, BYTE PTR [ebx]
			.if al != 0 
				;=============================
				; Yes there was a block here
				;=============================

				;=============================
				; Get the surface to use
				;=============================
				dec	eax
				shl	eax, 2
				mov	ebx, DWORD PTR BlockSurface[eax]

				;=============================
				; Calculate the Y coord
				;=============================
				mov	eax, ( GRID_HEIGHT - 5 )
				sub	eax, CurRow
				mov	ecx, BLOCK_HEIGHT
				mul	ecx
				mov	YPos, eax

				;=============================
				; Calculate the X coord
				;=============================
				mov	eax, CurCol
				mov	ecx, BLOCK_WIDTH
				mul	ecx
				add	eax, 251
				mov	XPos, eax

				;=============================
				; Blit the block
				;=============================
				DDS4INVOKE BltFast, lpddsback, XPos, YPos, \
					ebx, ADDR SrcRect, \
					DDBLTFAST_NOCOLORKEY or DDBLTFAST_WAIT

				;==============================
				; Did we succeed?
				;==============================
				.if eax == DDERR_SURFACELOST
					;======================
					; We lost the surface
					;======================


				.elseif eax != DD_OK
					;======================
					; We failed in some way
					;======================
					jmp	err

				.endif

			.endif

			;========================
			; Inc the Current Block
			;========================
			inc	CurBlock

			;========================
			; Incrment the Cur column
			;========================
			inc	CurCol

		.endw

		;====================
		; Incrment the row
		;====================
		inc	CurRow

	.endw

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Draw_Grid	ENDP
;########################################################################
; END Draw_Grid
;########################################################################

;########################################################################
; Draw_Captions Procedure
;########################################################################
Draw_Captions	proc	

	;=======================================================
	; This function will draw our captions, such as the 
	; score and the current level they are on
	;=======================================================

	;====================
	; Local Variables
	;====================
	LOCAL	hFont		:DWORD

	;=====================================
	; Get the DC for the back buffer
	;=====================================
	invoke DD_GetDC, lpddsback
	mov	hDC, eax

	;====================================
	; Set the font to "IMPACT" at the 
	; size that we need it
	;====================================
	invoke DD_Select_Font, hDC, -32, FW_BOLD, ADDR szImpact, ADDR Old_Obj
	mov	hFont, eax

	;=============================
	; Setup rect for score text
	;=============================
	mov	text_rect.top, 161
	mov	text_rect.left, 54
	mov	text_rect.right, 197
	mov	text_rect.bottom, 193

	;=============================
	; Draw the Score Text
	;=============================
	RGB 255, 255, 255
	push	eax
	mov	eax, Score
	mov	dwArgs, eax
	invoke wvsprintfA, ADDR szBuffer, ADDR szScore, Offset dwArgs
	pop	ebx
	invoke DD_Draw_Text, hDC, ADDR szBuffer, eax, ADDR text_rect,\
		DT_CENTER or DT_VCENTER or DT_SINGLELINE, ebx

	;=============================
	; Setup rect for Level text
	;=============================
	mov	text_rect.top, 67
	mov	text_rect.left, 102
	mov	text_rect.right, 151
	mov	text_rect.bottom, 99

	;=============================
	; Draw the Level Text
	;=============================
	RGB 255, 255, 0
	push	eax
	mov	eax, CurLevel
	mov	dwArgs, eax
	invoke wvsprintfA, ADDR szBuffer, ADDR szLevel, Offset dwArgs
	pop	ebx
	invoke DD_Draw_Text, hDC, ADDR szBuffer, eax, ADDR text_rect,\
		DT_CENTER or DT_VCENTER or DT_SINGLELINE, ebx

	;=============================
	; Setup rect for Lines text
	;=============================
	mov	text_rect.top, 256
	mov	text_rect.left, 90
	mov	text_rect.right, 162
	mov	text_rect.bottom, 288

	;=============================
	; Draw the Lines Text
	;=============================
	RGB 255, 255, 0
	push	eax
	mov	eax, NumLines
	mov	dwArgs, eax
	invoke wvsprintfA, ADDR szBuffer, ADDR szLines, Offset dwArgs
	pop	ebx
	invoke DD_Draw_Text, hDC, ADDR szBuffer, eax, ADDR text_rect,\
		DT_CENTER or DT_VCENTER or DT_SINGLELINE, ebx

	;=============================
	; Unselect the font
	;=============================
	invoke DD_UnSelect_Font, hDC, hFont, Old_Obj

	;============================
	; Release the DC
	;============================
	invoke DD_ReleaseDC, lpddsback, hDC

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Draw_Captions	ENDP
;########################################################################
; END Draw_Captions
;########################################################################

;########################################################################
; Rotate_Shape Procedure
;########################################################################
Rotate_Shape	proc	

	;=======================================================
	; This function will rotate the current shape it tests
	; to make sure there are no blocks already in the place
	; where it would rotate. 
	;=======================================================

	;================================
	; Local Variables
	;================================
	LOCAL	Index:		DWORD
	LOCAL	CurBlock:	DWORD
	LOCAL	Spot:		BYTE

	;=================================
	; Make sure they are low enough
	;=================================
	.if CurShapeY > 21
		jmp	err
	.endif

	;================================
	; Are they at the last frame?
	;================================
	.if CurShapeFrame == 3
		;=====================================
		; Yep ... make sure they can rotate
		;=====================================

		;========================================
		; We will start by seeing which half of 
		; the grid they are currently on that way
		; we know we much too move the shape 
		;=========================================
		.if CurShapeX < 6
			;===========================
			; They are on the left half 
			; of the grid
			;===========================

			;=============================
			; So start by moving them one 
			; coord right and saving the
			; old coordinat
			;=============================
			push	CurShapeX
			inc	CurShapeX

			;=============================
			; Now adjust the frame to what
			; it would be
			;=============================
			mov	CurShapeFrame, 0

			;=============================
			; Try to move them to the left
			;=============================
			invoke Move_Shape, MOVE_LEFT

			;=============================
			; If we succeeded then the old
			; X will be equal to the new
			; X coordinate
			;=============================
			mov	eax, CurShapeX
			pop	CurShapeX
			.if	eax == CurShapeX
				jmp	done
			.else
				;================
				; Can't rotate
				;================
				mov	CurShapeFrame, 3
				jmp	err

			.endif

		.else
			;===========================
			; They are on the right half 
			; of the grid
			;===========================

			;=============================
			; So start by moving them one 
			; coord left and saving the
			; old coordinat
			;=============================
			push	CurShapeX
			dec	CurShapeX

			;=============================
			; Now adjust the frame to what
			; it would be
			;=============================
			mov	CurShapeFrame, 0

			;=============================
			; Try & move them to the right
			;=============================
			invoke Move_Shape, MOVE_RIGHT

			;=============================
			; If we succeeded then the old
			; X will be equal to the new
			; X coordinate
			;=============================
			mov	eax, CurShapeX
			pop	CurShapeX
			.if	eax == CurShapeX
				;================
				; Can rotate
				;================
				jmp	done

			.else
				;================
				; Can't rotate
				;================
				mov	CurShapeFrame, 3
				jmp	err

			.endif

		.endif
		
	.else
		;=====================================
		; NO ... make sure they can rotate
		;=====================================

		;========================================
		; We will start by seeing which half of 
		; the grid they are currently on that way
		; we know we much too move the shape 
		;=========================================
		.if CurShapeX < 6
			;===========================
			; They are on the left half 
			; of the grid
			;===========================

			;=============================
			; So start by moving them one 
			; coord right and saving the
			; old coordinat
			;=============================
			push	CurShapeX
			inc	CurShapeX

			;=============================
			; Now adjust the frame to what
			; it would be
			;=============================
			inc	CurShapeFrame

			;=============================
			; Try to move them to the left
			;=============================
			invoke Move_Shape, MOVE_LEFT

			;=============================
			; If we succeeded then the old
			; X will be equal to the new
			; X coordinate
			;=============================
			mov	eax, CurShapeX
			pop	CurShapeX
			.if	eax == CurShapeX
				;================
				; Can rotate
				;================
				jmp	done

			.else
				;================
				; Can't rotate
				;================
				dec	CurShapeFrame
				jmp	err

			.endif

		.else
			;===========================
			; They are on the right half 
			; of the grid
			;===========================

			;=============================
			; So start by moving them one 
			; coord left and saving the
			; old coordinat
			;=============================
			push	CurShapeX
			dec	CurShapeX

			;=============================
			; Now adjust the frame to what
			; it would be
			;=============================
			inc	CurShapeFrame

			;=============================
			; Try & move them to the right
			;=============================
			invoke Move_Shape, MOVE_RIGHT

			;=============================
			; If we succeeded then the old
			; X will be equal to the new
			; X coordinate
			;=============================
			mov	eax, CurShapeX
			pop	CurShapeX
			.if	eax == CurShapeX
				;================
				; Can rotate
				;================
				jmp	done

			.else
				;================
				; Can't rotate
				;================
				dec	CurShapeFrame
				jmp	err

			.endif

		.endif

	.endif

done:
	;===================
	; Play the sound
	;===================
	invoke Play_Sound, Rotate_ID, 0

	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Rotate_Shape	ENDP
;########################################################################
; END Rotate_Shape
;########################################################################

;########################################################################
; Move_Shape Procedure
;########################################################################
Move_Shape	proc	Direction:DWORD

	;================================================
	; This function will move the shape in the 
	; desired direction
	;================================================

	;===========================
	; Local Variables
	;===========================
	LOCAL	CurCol:		DWORD
	LOCAL	CurRow:		DWORD
	LOCAL	CanMove:	DWORD

	;=================================
	; Make sure they are low enough
	;=================================
	.if CurShapeY > 21
		jmp	err
	.endif

	;====================================
	; Set CanMove to true it will
	; be false later if we can't move
	;====================================
	mov	CanMove, TRUE

	;================================================
	; Perform the Tests based on direction they want
	;================================================
	.if Direction == MOVE_LEFT
		;====================================
		; They want to move to the left
		;====================================
		
		;====================================
		; Find the Left most column with a
		; valid block inside of it
		;====================================
		mov	CurCol, 0
		.while CurCol < 4
			;==========================
			; Calculate Our Mask
			;==========================
			mov	eax, 1
			mov	ecx, 3	; Start from the Left
			sub	ecx, CurCol
			shl	eax, cl
			mov	edx, eax
			push	edx

			;===========================
			; Go through all 4 rows
			;===========================
			mov	CurRow, 0
			.while CurRow < 4
				;===============================
				; Get the Current Line of Blocks
				;===============================
				mov	ebx, CurShape
				mov	eax, CurShapeFrame
				shl	eax, 2
				add	ebx, eax
				add	ebx, CurRow
				xor	ecx, ecx
				mov	cl, BYTE PTR [ebx]

				;========================
				; Test the Mask and the
				; current line of blocks
				;========================
				pop	edx
				push	edx
				.if ( edx & ecx )
					;====================
					; There was a Block
					;====================

					;====================
					; Calculate the 
					; block's X value
					;====================
					mov	eax, CurShapeX
					add	eax, CurCol

					;====================
					; Can we move?
					;====================
					.if eax == 0 || eax > 100
						;=============
						; Nope
						;=============
						mov	CanMove, FALSE

					.else
						;========================
						; Calculate the block to 
						; the left of us
						;========================
						mov	eax, CurShapeY
						sub	eax, CurRow
						mov	ecx, 13
						mul	ecx
						add	eax, CurShapeX
						add	eax, CurCol
						dec	eax	; 1 to the Left
						mov	ecx, BlockGrid
						add	ecx, eax
						mov	al, BYTE PTR [ecx]
								
						;======================
						; Are we blocked?
						;======================
						.if al != 0
							;================
							; We are blocked
							;================
							mov	CanMove, FALSE
						
						.endif

					.endif
				
				.endif

				;===========================
				; Inc our current row
				;===========================
				inc	CurRow

			.endw

			;===========================
			; Clean Off the stack
			;===========================
			pop	edx

			;===========================
			; Inc our current column
			;===========================
			inc	CurCol

		.endw

		;==================================
		; Can we Still Move
		;==================================
		.if CanMove == TRUE
			;=======================
			; yes we can
			;=======================
			dec	CurShapeX
	
		.endif

	.elseif Direction == MOVE_RIGHT
		;====================================
		; They want to move to the right
		;====================================

		;====================================
		; Find the Right most column with a
		; valid block inside of it
		;====================================
		mov	CurCol, 4
		.while CurCol > 0
			;==========================
			; Calculate Our Mask
			;==========================
			mov	eax, 1
			mov	ecx, 4	; Start from the Right
			sub	ecx, CurCol
			shl	eax, cl
			mov	edx, eax
			push	edx
			
			;===========================
			; Go through all 4 rows
			;===========================
			mov	CurRow,0
			.while CurRow < 4
				;===============================
				; Get the Current Line of Blocks
				;===============================
				mov	ebx, CurShape
				mov	eax, CurShapeFrame
				shl	eax, 2
				add	ebx, eax
				add	ebx, CurRow
				xor	ecx, ecx
				mov	cl, BYTE PTR [ebx]

				;========================
				; Test the Mask and the
				; current line of blocks
				;========================
				pop	edx
				push	edx
				.if ( edx & ecx )
					;====================
					; There was a Block
					;====================

					;====================
					; Calculate the 
					; block's X value
					;====================
					mov	eax, CurShapeX
					add	eax, CurCol
					dec	eax

					;====================
					; Can we move?
					;====================
					.if eax >= 12 
						;=============
						; Nope
						;=============
						mov	CanMove, FALSE

					.else
						;========================
						; Calculate the block to 
						; the right of us
						;========================
						mov	eax, CurShapeY
						sub	eax, CurRow
						mov	ecx, 13
						mul	ecx
						add	eax, CurShapeX
						add	eax, CurCol	; Already 1 to the
									; Right
						mov	ecx, BlockGrid
						add	ecx, eax
						mov	al, BYTE PTR [ecx]

						;======================
						; Are we blocked?
						;======================
						.if al != 0 
							;================
							; We are blocked
							;================
							mov	CanMove, FALSE
						
						.endif

					.endif

				.endif

				;===========================
				; Inc our current row
				;===========================
				inc	CurRow

			.endw

			;===========================
			; Clean Off the stack
			;===========================
			pop	edx

			;===========================
			; dec our current column
			;===========================
			dec	CurCol

		.endw

		;==================================
		; Can we Still Move
		;==================================
		.if CanMove == TRUE
			;=======================
			; yes we can
			;=======================
			inc	CurShapeX
	
		.endif

	.elseif Direction == MOVE_DOWN
		;====================================
		; They want to move the piece down
		;====================================

		;====================================
		; Test for a collision
		;====================================
		invoke Test_Collision
		.if eax == FALSE
			;============================
			; It is safe to drop a notch
			;============================
			dec	CurShapeY

		.endif

	.else
		;====================================
		; They passed an invalid direction
		;====================================
		jmp	err

	.endif

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Move_Shape	ENDP
;########################################################################
; END Move_Shape
;########################################################################

;########################################################################
; Update_Shape Procedure
;########################################################################
Update_Shape	proc

	;================================================
	; This function will update our shape ... or 
	; drop it down by a grid notch and test for
	; a collision with the grid
	;================================================
	
	;========================
	; Can we move down???
	;========================
	invoke Test_Collision
	.if eax == TRUE
		;=======================
		; NO... we hit something
		;=======================

		;=============================
		; Place the piece in the grid
		;=============================
		invoke Place_In_Grid

		;=========================
		; Jmp & Return with False
		;=========================
		jmp	err

	.else
		;===========================
		; yes we can drop down
		;===========================

		;=================================
		; Drop our piece down by a notch
		;=================================
		dec	CurShapeY

	.endif

done:
	;===================
	; We hit nothing
	;===================
	return TRUE

err:
	;===================
	; We hit something
	;===================
	return FALSE

Update_Shape	ENDP
;########################################################################
; END Update_Shape
;########################################################################

;########################################################################
; New_Shape Procedure
;########################################################################
New_Shape	proc	

	;================================================
	; This function will select a new shape at random
	; for the Next shape and assign the old next
	; shape values to the current shape
	;================================================

	;=================================
	; Do the swaps if this isn't our
	; very first piece of the game
	;=================================
again:
	.if NextShape != -1
		m2m	CurShape, NextShape
		m2m	CurShapeColor, NextShapeColor
		m2m	CurShapeX, NextShapeX
		m2m	CurShapeY, NextShapeY
		m2m	CurShapeFrame, NextShapeFrame
	.endif

	;======================================
	; First make sure they haven't reached
	; the top of the grid yet
	;
	; Begin by calculating the start of 
	; the very last row where the piece
	; is initialized at ... aka (5,19)
	;======================================
	mov	eax, 13
	mov	ecx, 19
	mul	ecx
	add	eax, 5
	mov	ebx, BlockGrid
	add	eax, ebx
	mov	ecx, eax
	add	ecx, 4

	;==========================
	; Loop through and test the
	; next 4 positions
	;==========================
	.while eax <= ecx
		;=====================
		; Is this one filled?
		;=====================
		mov	bl, BYTE PTR [eax]
		.if bl != 0
			;===================
			; They are dead
			;===================
			jmp	err

		.endif

		;=================
		; Inc the counter
		;=================
		inc	eax
	.endw

	;=============================
	; Use a random number to get
	; the current shape to use
	;
	; For this we will just use 
	; the time returned by the 
	; Get_Time() function
	;=============================
	invoke Get_Time

	;=============================
	; Mod this number with 7
	; since there are 7 shapes
	;=============================
	mov	ecx, 7
	xor	edx, edx
	div	ecx
	mov	eax, edx

	;=============================
	; Multiply by 16 since there
	; are 16 bytes per shape
	;=============================
	shl	eax, 4

	;=============================
	; Use that number to select  
	; the shape from the table
	;=============================
	mov	ebx, offset ShapeTable
	add	eax, ebx
	mov	NextShape, eax

	;=============================
	; Use a random number to get
	; the block surface to use
	;
	; For this we will just use 
	; the time returned by the 
	; Get_Time() function
	;=============================
	invoke Get_Time

	;=============================
	; And this result with 7
	; since there are 8 blocks
	;=============================
	and	eax, 7

	;================================
	; Use it as the block surface
	;================================
	mov	NextShapeColor, eax

	;================================
	; Initialize the Starting Coords
	;================================
	mov	NextShapeX, 5
	mov	NextShapeY, 24

	;================================
	; Set the Current Frame Variable
	;================================
	mov	NextShapeFrame, 0

	;====================================
	; Go back to the top and load again
	; if this was our very first piece
	;====================================
	.if CurShape == -1
		jmp	again
	.endif

done:
	;=======================
	; They have a new piece
	;=======================
	return TRUE

err:
	;===================
	; They died!
	;===================
	return FALSE

New_Shape	ENDP
;########################################################################
; END New_Shape
;########################################################################

;########################################################################
; Place_In_Grid Procedure
;########################################################################
Place_In_Grid	proc	

	;================================================
	; This function will place the current shape 
	; into the grid
	;================================================

	;===========================
	; Local Variables
	;===========================
	LOCAL	DrawY:		DWORD
	LOCAL	DrawX:		DWORD
	LOCAL	CurRow:		DWORD
	LOCAL	CurCol:		DWORD
	LOCAL	CurLine:	DWORD
	LOCAL	CurGrid:	DWORD

	;===================================
	; Get the Current Shape Pos
	;===================================
	mov	ebx, CurShape
	mov	eax, CurShapeFrame
	shl	eax, 2
	add	ebx, eax
	mov	CurLine, ebx

	;===================================
	; Set the Starting Row and Column
	; for the placement of the block
	;===================================
	mov	eax, CurShapeX
	mov	ebx, CurShapeY
	mov	DrawX, eax
	mov	DrawY, ebx

	;===================================
	; Loop through all four rows
	;===================================
	mov	CurRow, 0
	.while CurRow < 4
		;=====================================
		; Loop through all four Columns 
		;=====================================
		mov	CurCol, 4
		.while CurCol > 0 
			;===============================
			; Shift the CurLine Byte over
			; by our CurCol
			;===============================
			mov	ecx, 4
			sub	ecx, CurCol
			mov	ebx, CurLine
			xor	eax, eax
			mov	al, BYTE PTR [ebx]
			shr	eax, cl

			;===============================
			; Is it a valid block?
			;===============================
			.if ( eax & 1 )
				;============================
				; Yes it was a valid block
				;============================
	
				;=============================
				; Calculate the Block in our
				; BlockGrid to place it in
				;=============================
				mov	eax, DrawY
				mov	ecx, 13
				mul	ecx
				mov	ebx, DrawX
				add	ebx, CurCol
				dec	ebx
				add	eax, ebx
				mov	ecx, BlockGrid
				add	eax, ecx

				;=============================
				; Store the Color in the Block
				; add one since we let 0 mean
				; the block is empty
				;=============================
				mov	ebx, CurShapeColor
				inc	ebx
				mov	BYTE PTR [eax], bl

			.endif

			;=====================
			; Dec our col counter
			;=====================
			dec	CurCol

		.endw

		;=======================
		; Inc the CurLine
		;=======================
		inc	CurLine

		;====================
		; decrement Y coord
		;====================
		dec	DrawY

		;====================
		; Inc the row counter
		;====================
		inc	CurRow

	.endw

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Place_In_Grid	ENDP
;########################################################################
; END Place_In_Grid
;########################################################################

;########################################################################
; Test_Collision Procedure
;########################################################################
Test_Collision	proc	

	;================================================
	; This function will test for a collision between
	; the grid and the current shape
	;================================================

	;==============================
	; Local Variables
	;==============================
	LOCAL	Index:	DWORD
	LOCAL	Adjust:	DWORD

	;========================================
	; Loop through and find the first block
	; in each of the four columns
	;
	; NOTE: 0 = RIGHT 3 = LEFT
	;========================================
	mov	Index, 0
	.while Index < 4
		;==========================================
		; Start at the bottom of the Current Frame
		;==========================================
		mov	ebx, CurShape
		mov	eax, CurShapeFrame
		shl	eax, 2
		add	ebx, eax
		add	ebx, 3

		;=========================================
		; Now loop until we have a one in the 
		; current colum we are working on or we 
		; reach the top
		;==========================================
		mov	Adjust, 4
		.while	Adjust > 0
			;=======================
			; Get the Current Line
			;=======================
			xor	eax, eax
			mov	al, BYTE PTR [ebx]

			;=======================
			; Adjust by the Column
			;=======================
			mov	ecx, Index
			shr	eax, cl

			;=========================
			; Was there a block there
			;=========================
			.if ( eax & 1 )
				;======================
				; Yes there was a block
				;======================

				;=============================
				; Have we hit Bottom
				;=============================
				mov	eax, CurShapeY
				sub	eax, Adjust
				inc	eax		; Off by 1 syndrome
				.if eax == 0
					;================
					; Bottom of grid
					;================
					jmp	done

				.endif

				;===========================
				; Calculate the Block right 
				; under it on the grid
				;===========================
				dec	eax		; Move Under it
				mov	ecx, 13
				mul	ecx
				add	eax, CurShapeX
				add	eax, 3
				sub	eax, Index
				mov	ecx, BlockGrid
				add	ecx, eax

				;===========================
				; Does the Block have one
				; underneath it on the grid?
				;===========================
				mov	al, BYTE PTR [ecx]
				.if al != 0
					;===========================
					; We had a valid collision
					;===========================
					jmp	done
			
				.endif

			.endif
		
			;=================================
			; No Block -- Previous Line Please
			;=================================
			dec	ebx

			;===============================
			; Decrement the Adjust counter
			;===============================
			dec	Adjust

		.endw

		;==================================
		; Next Column Please!
		;==================================
		inc	Index

	.endw

err:
	;===================
	; We didn't collide
	;===================
	return FALSE

done:
	;===================
	; Play the sound
	;===================
	invoke Play_Sound, Drop_ID, 0

	;===================
	; We collided
	;===================
	return TRUE

Test_Collision	ENDP
;########################################################################
; END Test_Collision
;########################################################################

;########################################################################
; Line_Test Procedure
;########################################################################
Line_Test	proc	

	;================================================
	; This function will test to see if they earned a
	; line ... if so it will eliminate that line
	; and update our grid of blocks
	;================================================

	;==========================
	; Local Variables
	;==========================
	LOCAL	CurLine:	DWORD
	LOCAL	CurBlock:	DWORD

	;===============================
	; Start at the Base of the Grid
	;===============================
	mov	CurLine, 0

	;=================================
	; Loop through all possible Lines
	;=================================
	.while CurLine < (GRID_HEIGHT - 4)
		;===================================
		; Goto the base of the current line
		;===================================
		mov	eax, CurLine
		mov	ecx, 13
		mul	ecx
		add	eax, BlockGrid

		;==================================
		; Loop through every block
		; testing to see if it is valid
		;==================================
		mov	CurBlock, 0
		.while CurBlock < (GRID_WIDTH)
			;==========================
			; Is this Block IN-Valid?
			;==========================
			mov	bl, BYTE PTR [eax]
			.if bl == 0
				;===================
				; Yes, so break
				;===================
				.break

			.endif

			;======================
			; Next Block 
			;======================
			inc	eax

			;======================
			; Inc the counter
			;======================
			inc	CurBlock

		.endw

		;==============================
		; Did our inner loop go all
		; of the way through??
		;==============================
		.if CurBlock == (GRID_WIDTH)
			;============================
			; Yes. That means that it was 
			; a valid line we just earned
			;============================

			;===================================
			; Calculate How much memory to move
			; TOTAL - Amount_IN = TO_MOVE
			;===================================
			mov	ebx, (GRID_WIDTH * (GRID_HEIGHT -5))
			mov	eax, CurLine
			mov	ecx, 13
			mul	ecx
			push	eax
			sub	ebx, eax

			;============================
			; Move the memory one line
			; up to our current line
			;============================
			pop	eax
			add	eax, BlockGrid
			mov	edx, eax
			add	edx, 13

			;==============================
			; Move the memory down a notch
			;==============================
			invoke RtlMoveMemory, eax, edx, ebx

			;============================
			; Jump down and return TRUE
			;============================
			jmp	done

		.endif

		;==============================
		; Incrment our Line counter
		;==============================
		inc	CurLine

	.endw

err:
	;===================
	; We didn't get one
	;===================
	return FALSE

done:
	;===================
	; Play the sound
	;===================
	invoke Play_Sound, Thud_ID, 0

	;==========================
	; Adjust their line count
	;==========================
	inc	NumLines
	.if NumLines >= MAX_LINES
		mov	NumLines, 0
		inc	CurLevel
	.endif

	;===================
	; We earned a line
	;===================
	return TRUE


Line_Test	ENDP
;########################################################################
; END Line_Test
;########################################################################

;########################################################################
; Adjust_Score Procedure
;########################################################################
Adjust_Score	proc	amount:DWORD

	;================================================
	; This function will adjust the score by the 
	; passed in value if possible, adjusting the
	; level if necessary
	;================================================
	mov	eax, amount
	add	Score, eax
	.if Score > MAX_SCORE
		mov	Score, MAX_SCORE
	.endif

done:
	;===================
	; We earned a line
	;===================
	return TRUE


Adjust_Score	ENDP
;########################################################################
; END Adjust_Score
;########################################################################

;########################################################################
; Is_Game_Won Procedure
;########################################################################
Is_Game_Won	proc

	;================================================
	; This function will return TRUE if we have won
	; the game and false otherwise
	;================================================

	.if CurLevel > MAX_LEVEL
		return TRUE
	.else
		return FALSE
	.endif

Is_Game_Won	ENDP
;########################################################################
; END Is_Game_Won
;########################################################################

;######################################
; THIS IS THE END OF THE PROGRAM CODE #
;######################################
end 

