;###########################################################################
;###########################################################################
; ABOUT Trans:
;
;	This code module contains all of the functions that perform
; transitions. There is one main function that is provided as an
; interface to all of the transitions. They are called at random by 
; the main function.
;
;	Of course, you are free to call any of the transitions yourself,
; I just wanted to keep things random. Right now there is only 1 transition
; with 2 options. Add more as you please.
;
;###########################################################################
;###########################################################################

;###########################################################################
;###########################################################################
; THE COMPILER OPTIONS
;###########################################################################
;###########################################################################

	.386
	.model flat, stdcall
	option casemap :none   ; case sensitive

;###########################################################################
;###########################################################################
; THE INCLUDES SECTION
;###########################################################################
;###########################################################################

	;================================================
	; These are the Inlcude files for Window stuff
	;================================================
	include \masm32\include\windows.inc
	include \masm32\include\comctl32.inc
	include \masm32\include\comdlg32.inc
	include \masm32\include\shell32.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc
	
	;===============================================
	; The Lib's for those included files
	;================================================
	includelib \masm32\lib\comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib

	;====================================
	; The Direct Draw include file
	;====================================
	include Includes\DDraw.inc

	;=================================================
	; Include the file that has our protos
	;=================================================
	include Protos.inc

;###########################################################################
;###########################################################################
; LOCAL MACROS
;###########################################################################
;###########################################################################

	m2m MACRO M1, M2
		push		M2
		pop		M1
	ENDM

	return MACRO arg
		mov	eax, arg
        	ret
	ENDM

;#################################################################################
;#################################################################################
; External variables
;#################################################################################
;#################################################################################

	;=================================
	; The DirectDraw stuff
	;=================================
	EXTERN	lpddsprimary	:LPDIRECTDRAWSURFACE4
	EXTERN	lpddsback	:LPDIRECTDRAWSURFACE4

	;=================================
	; The app width/height
	;=================================
	EXTERN	app_width	:DWORD
	EXTERN	app_height	:DWORD

;#################################################################################
;#################################################################################
; BEGIN INITIALIZED DATA
;#################################################################################
;#################################################################################

    .data

	;=========================================
	; Rectangles for the different blits
	;=========================================
	SrcRect		RECT <0,0,0,0>
	DestRect	RECT <0,0,0,0>

;#################################################################################
;#################################################################################
; BEGIN CONSTANTS
;#################################################################################
;#################################################################################


;#################################################################################
;#################################################################################
; BEGIN EQUATES
;#################################################################################
;#################################################################################

	;=================
	;Utility Equates
	;=================
FALSE		equ	0
TRUE		equ	1

	;==========================
	; The sync time for each
	; of the transitions
	;==========================
TRANS_TIME	equ	30	; 33 FPS

	;===============================
	; These are used by Wipe_Trans
	; to indicate the direction
	;===============================
WIPE_HORZ	equ	0
WIPE_VERT	equ	1

;#################################################################################
;#################################################################################
; BEGIN THE CODE SECTION
;#################################################################################
;#################################################################################

  .code

;########################################################################
; Transition Procedure
;########################################################################
Transition	proc

	;=======================================================
	; This function will call one of the our transitions
	; based on a random number. All transitions require
	; the primary buffer to be the surface that you want
	; to transition from and the back buffer to be the
	; surface you want to transition to. Both need to
	; be unlocked.
	;=======================================================

	;=============================
	; Get a random number
	;=============================
	invoke Get_Time

	;=============================
	; Mod the result with 2
	;=============================
	and	eax, 1

	;=============================
	; Select the transition based
	; on our number
	;=============================
	.if eax == 0
		;==========================
		; Perform a Horizontal Wipe
		;==========================
		invoke Wipe_Trans, 6, WIPE_HORZ

		;=========================
		; Universal error check
		;=========================
		.if eax == FALSE
			jmp	err
		.endif

	.elseif eax == 1
		;==========================
		; Perform a Vertical Wipe
		;==========================
		invoke Wipe_Trans, 4, WIPE_VERT

		;=========================
		; Universal error check
		;=========================
		.if eax == FALSE
			jmp	err
		.endif


	.endif

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Transition	ENDP
;########################################################################
; END Transition
;########################################################################

;########################################################################
; Wipe_Trans Procedure
;########################################################################
Wipe_Trans	proc	strip_width:DWORD, direction:DWORD

	;=======================================================
	; This function will perform either a horizontal or
	; a vertical wipe depending on what you pass in for the
	; direction paramter. The width of each step is 
	; determined by the width that you pass in to it.
	;=======================================================

	;=========================================
	; Local Variables
	;=========================================
	LOCAL	StartTime	:DWORD

	;========================================
	; Setup the source rectangle and the 
	; destination rectangle
	;
	; For the first iteration, the strip may 
	; not be the height passed in. This is to 
	; make sure we are on an even boundary 
	; during the loop below
	;========================================
	.if direction == WIPE_HORZ
		mov	SrcRect.top, 0
		mov	SrcRect.left, 0
		mov	eax, app_width
		mov	ecx, strip_width
		xor	edx, edx
		div	ecx
		.if edx == 0
			mov	edx, strip_width
		.endif
		mov	ebx, app_height
		mov	SrcRect.bottom, ebx
		mov	SrcRect.right, edx
		mov	DestRect.top, 0
		mov	DestRect.left, 0
		mov	DestRect.bottom, ebx
		mov	DestRect.right, edx

	.elseif direction == WIPE_VERT
		mov	SrcRect.top, 0
		mov	SrcRect.left, 0
		mov	eax, app_height
		mov	ecx, strip_width
		xor	edx, edx
		div	ecx
		mov	eax, app_width
		.if edx == 0
			mov	edx, strip_width
		.endif
		mov	SrcRect.bottom, edx
		mov	SrcRect.right, eax
		mov	DestRect.top, 0
		mov	DestRect.left, 0
		mov	DestRect.bottom, edx
		mov	DestRect.right, eax

	.else
		;==================
		; Invalid direction
		;==================
		jmp	err

	.endif

	;================================
	; Get the starting time
	;================================
	invoke Start_Time, ADDR StartTime

	;================================
	; Blit the strip onto the screen
	;================================
	DDS4INVOKE BltFast, lpddsprimary, SrcRect.left, SrcRect.top,\
		lpddsback, ADDR DestRect, DDBLTFAST_WAIT

	;===============================
	; Make sure we succeeded
	;===============================
	.if eax != DD_OK
		jmp	err
	.endif

	;===================================================
	; Now adjust the distance between the left &
	; right, or top and bottom, so that the top, or
	; left, corner is where the right hand side was 
	; at ... and the bottom, or right, is strip_width
	; away from the opposite corner.
	;===================================================
	mov	eax, strip_width
	.if direction == WIPE_HORZ
		mov	ebx, SrcRect.right
		mov	SrcRect.left, ebx
		mov	DestRect.left, ebx
		add	ebx, eax
		mov	DestRect.right, ebx
		mov	SrcRect.right, ebx

	.elseif direction == WIPE_VERT
		mov	ebx, SrcRect.bottom
		mov	SrcRect.top, ebx
		mov	DestRect.top, ebx
		add	ebx, eax
		mov	DestRect.bottom, ebx
		mov	SrcRect.bottom, ebx

	.endif

	;===================================
	; Wait to synchronize the time
	;===================================
	invoke Wait_Time, StartTime, TRANS_TIME

	;=====================================
	; Drop into a while loop and blit all
	; of the strips synching to our 
	; desired transition rate
	;=====================================
	.while TRUE
		;================================
		; Get the starting time
		;================================
		invoke Start_Time, ADDR StartTime

		;================================
		; Blit the strip onto the screen
		;================================
		DDS4INVOKE BltFast, lpddsprimary, SrcRect.left, SrcRect.top,\
			lpddsback, ADDR DestRect, DDBLTFAST_WAIT

		;===============================
		; Make sure we succeeded
		;===============================
		.if eax != DD_OK
			jmp	err
		.endif

		;==================================
		; Have we reached our extents yet
		;==================================
		mov	eax, SrcRect.bottom 
		mov	ebx, app_height
		mov	ecx, SrcRect.right
		mov	edx, app_width
		.if eax == ebx && ecx == edx
			;======================
			; Trans complete
			;======================
			.break

		.else
			;======================
			; Adjust by the strip
			;======================
			mov	eax, strip_width
			.if direction == WIPE_HORZ
				add	SrcRect.left, eax
				add	SrcRect.right, eax
				add	DestRect.left, eax
				add	DestRect.right, eax

			.elseif direction == WIPE_VERT
				add	SrcRect.top, eax
				add	SrcRect.bottom, eax
				add	DestRect.top, eax
				add	DestRect.bottom, eax

			.endif

		.endif

		;===================================
		; Wait to synchronize the time
		;===================================
		invoke Wait_Time, StartTime, TRANS_TIME

	.endw	

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Wipe_Trans	ENDP
;########################################################################
; END Wipe_Trans
;########################################################################

;######################################
; THIS IS THE END OF THE PROGRAM CODE #
;######################################
end 

